#include "CMQObject.h"
#include <algorithm>

using namespace std;

const float CMQCoordinate::uvdelta = 0.0001f;

CMQObject &CMQDocument::insertObject(CMQObject &newobj, const CMQObject &before)
{
	int id = doc_->InsertObject(newobj.obj_, before.obj_);
	newobj.obj_->SetDepth(before.obj_->GetDepth());
	newobj.obj_->SetVisible(before.obj_->GetVisible());
	return newobj;
}

CMQObject CMQDocument::getCurrentObject() {
	int idx = doc_->GetCurrentMaterialIndex();
	return CMQObject(doc_->GetObject(idx));
}

CMQObject CMQObject::clone()
{
	return CMQObject(obj_->Clone());
}

/*
	MQObject̖O擾stringŕԂ
*/
string CMQObject::getName()
{
	const int NAME_BUFFER_SIZE = 250;
	char objname[NAME_BUFFER_SIZE];
	obj_->GetName(objname, NAME_BUFFER_SIZE);
	return string(objname);
}

void CMQObject::setName(const std::string &name)
{
	obj_->SetName(name.c_str());
}

/*
	namex[XƂĎgĂȂO擾stringŕԂ
*/
string CMQDocument::getUnusedName(const string &name)
{
	const int NAME_BUFFER_SIZE = 250;
	char unusedname[NAME_BUFFER_SIZE];
	doc_->GetUnusedObjectName(unusedname, NAME_BUFFER_SIZE, name.c_str());
	return string(unusedname);
}

CMQObject CMQDocument::getObjectbyName(const string &name)
{
	for(int i = 0; i < doc_->GetObjectCount(); i++) {
		CMQObject obj = getObject(i);
		if(obj.getName() == name) {
			return obj;
		}
	}
	return CMQObject();
}

vector<int> CMQObject::getVertexRelatedFaces(int vindex)
{
	vector<int> faces;
	int fcount = obj_->GetVertexRelatedFaces(vindex, nullptr);
	if(fcount == 0) {
		return faces;
	}
	unique_ptr<int[]> facearray(new int[fcount]);
	obj_->GetVertexRelatedFaces(vindex, facearray.get());
	faces.reserve(fcount);
	for(int i = 0; i < fcount; i++) {
		faces.push_back(facearray[i]);
	}
	return faces;
}

vector<int> CMQObject::getFacePointArray(int findex)
{
	vector<int> vertexes;
	int vcount = obj_->GetFacePointCount(findex);
	if(vcount == 0) {
		return vertexes;
	}
	unique_ptr<int[]> vertexarray(new int[vcount]);
	obj_->GetFacePointArray(findex, vertexarray.get());
	vertexes.reserve(vcount);
	for(int i = 0; i < vcount; i++) {
		vertexes.push_back(vertexarray[i]);
	}
	return vertexes;
}

vector<CMQCoordinate> CMQObject::getFaceCoordinateArray(int findex)
{
	vector<CMQCoordinate> cood;
	int ccount = obj_->GetFacePointCount(findex);
	if(ccount == 0) {
		return cood;
	}
	unique_ptr<int[]> vertexarray(new int[ccount]);
	obj_->GetFacePointArray(findex, vertexarray.get());
	unique_ptr<MQCoordinate[]> coodarry(new MQCoordinate[4]);
	obj_->GetFaceCoordinateArray(findex, coodarry.get());
	cood.reserve(ccount);
	for(int i = 0; i < ccount; i++) {
		cood.push_back(CMQCoordinate(vertexarray[i], coodarry[i]));
	}
	return cood;
}

void CMQObject::setFaceCoordinateArray(int findex, int vertex, const CMQCoordinate &co)
{
	int vcount = obj_->GetFacePointCount(findex);
	unique_ptr<MQCoordinate[]> uvarray(new MQCoordinate[4]);
	obj_->GetFaceCoordinateArray(findex, uvarray.get());
	uvarray[vertex] = co.getCoordinate();
	obj_->SetFaceCoordinateArray(findex, uvarray.get());
}
